@@ -1,16 +1,55 @@
 # -*- coding: utf-8 -*-
 """
-Given a database url,
-create ApiLogicServer project by cloning prototype,
-in particular create the ui/basic_web_app/app/views.py
-and api/expose_api_models.
+Given a database url, create [and run] ApiLogicProject.
 
-See: main driver
+Basically clone prototype project, and create:
+* database/models.py
+* ui/admin/admin.yaml
+* api/expose_api_models
+
+Special provisions for NW Sample, to show customizations.
+
+See end for key module map quick links.
 
 """
 
-__version__ = "3.50.19"
+__version__ = "4.00.06"
 
+recent_changes = \
+    f'\n\nRecent Changes:\n' +\
+    "\t12/26/2021 - 04.00.06: Performance enhancement to list screen \n"\
+    "\t12/26/2021 - 04.00.05: Introducing the admin app, with Readme Tutorial \n"\
+    "\t11/13/2021 - 03.50.01: rebuild-from-database/model, improved relationship support, port conflict msg \n"\
+    "\t11/04/2021 - 03.40.01: Per MacOS Monterey, default ports to 5001, 5002 \n"\
+    "\t09/29/2021 - 03.01.15: run (now just runs without create), added create-and-run \n"\
+    "\t09/15/2021 - 03.00.09: auto-create .devcontainer for vscode, configure network, python & debug \n"\
+    "\t09/10/2021 - 03.00.02: rename logic_bank to declare_logic, improved logging\n"
+
+uri_info = [
+    '',
+    'Creates and optionally runs a customizable ApiLogicServer project',
+    '',
+    'Examples:',
+    '  ApiLogicServer create-and-run',
+    '  ApiLogicServer create-and-run --db_url=sqlite:///nw.sqlite',
+    '  ApiLogicServer create-and-run --db_url=mysql+pymysql://root:p@mysql-container:3306/classicmodels '
+    '--project_name=/localhost/docker_db_project',
+    '  ApiLogicServer create-and-run --db_url=mssql+pyodbc://sa:posey386!@localhost:1433/NORTHWND?'
+    'driver=ODBC+Driver+17+for+SQL+Server?trusted_connection=no',
+    '  ApiLogicServer create-and-run --db_url=postgresql://postgres:p@10.0.0.234/postgres',
+    '  ApiLogicServer create-and-run --db_url=postgresql+psycopg2:'
+    '//postgres:password@localhost:5432/postgres?options=-csearch_path%3Dmy_db_schema',
+    '  ApiLogicServer create --project_name=Chinook \\',
+    '    --host=ApiLogicServer.pythonanywhere.com --port= \\',
+    '    --db_url=mysql+pymysql://ApiLogicServer:***@ApiLogicServer.mysql.pythonanywhere-services.com/ApiLogicServer\$Chinook',
+    '',
+    'Where --db_url defaults to supplied sample, or, specify URI for your own database:',
+    '   SQLAlchemy Database URI help: https://docs.sqlalchemy.org/en/14/core/engines.html',
+    '   Other URI examples:           https://github.com/valhuber/ApiLogicServer/wiki/Testing',
+    '   Install Guide:                https://github.com/valhuber/ApiLogicServer/wiki/Install-Guide',
+    ' ',
+    'Docs: https://github.com/valhuber/ApiLogicServer#readme\n'
+]
 from contextlib import closing
 
 import yaml
@@ -61,12 +100,13 @@ api_logic_server_info_file = open(api_logic_server_info_file_name)
 api_logic_server_info_file_dict = yaml.load(api_logic_server_info_file, Loader=yaml.FullLoader)
 api_logic_server_info_file.close()
 
+
 last_created_project_name = api_logic_server_info_file_dict["last_created_project_name"]
-default_db = "<default -- nw.sqlite>"
-default_project_name = "api_logic_server"
+default_db = "default = nw.sqlite, ? for help"
+default_project_name = "ApiLogicProject"
 default_fab_host = "localhost"
 if os.path.exists('/home/api_logic_server'):  # docker?
-    default_project_name = "/localhost/api_logic_server"
+    default_project_name = "/localhost/ApiLogicProject"
     default_fab_host = "0.0.0.0"
 nw_db_status = ""
 """ '', nw, nw+, nw- """
@@ -270,7 +310,7 @@ def clone_prototype_project_with_nw_samples(project_directory: str, project_name
             code_loc = str(get_api_logic_server_dir())
             from_dir = (Path(code_loc)).\
                 joinpath('project_prototype')  # /Users/val/dev/ApiLogicServer/project_prototype
-        print(f'{msg} copy {from_dir} -> {project_directory}')
+        print(f'{msg} copy {from_dir} -> {os.path.realpath(project_directory)}')
         cloned_from = from_dir
         try:
             if project_name != ".":
@@ -336,6 +376,8 @@ def clone_prototype_project_with_nw_samples(project_directory: str, project_name
 
         print(f'.. ..Copied sqlite db to: {target_db_loc_actual} and '
               f'updated db_uri in {project_directory}/config.py')
+        if nw_db_status != "":
+            print(f'.. .. ..Using nw sample db: {db_loc}')
 
     return target_db_loc_actual
 
@@ -609,20 +651,20 @@ def start_open_with(open_with: str, project_name: str):
 
 
 def is_docker() -> bool:
-    """ running docker?  path exists: /home/api_logic_server
-    """
+    """ running docker?  dir exists: /home/api_logic_server """
     path = '/home/api_logic_server'
-    return (
-        os.path.exists('/.dockerenv') or
-        os.path.isfile(path) and any('docker' in line for line in open(path))
-    )
+    return os.path.isdir(path)
 
 
 def print_options(project_name: str, db_url: str, host: str, port: str, not_exposed: str,
                   from_git: str, db_types: str, open_with: str, run: bool, use_model: str, admin_app: bool,
                   flask_appbuilder: bool, favorites: str, non_favorites: str, react_admin:bool,
                   extended_builder: str):
-    """ Creating ApiLogicServer with options:"""
+    """ Creating ApiLogicServer with options: (or uri helo) """
+    if db_url == "?":
+        print_uri_info()
+        exit(0)
+
     print_options = True
     if print_options:
         print(f'\n\nCreating ApiLogicServer with options:')
@@ -710,13 +752,15 @@ def api_logic_server(project_name: str, db_url: str, host: str, port: str, not_e
                   flask_appbuilder=flask_appbuilder, favorites=favorites, non_favorites=non_favorites,
                   react_admin=react_admin, admin_app=admin_app,
                   extended_builder=extended_builder)
+
     print(f"\nApiLogicServer {__version__} Creation Log:")
 
     nw_db_status = ""  # presume not northwind
     abs_db_url = db_url
     """ non-relative db location - we work with this (but NB: we copy sqlite db to <project>/database) """
     if db_url in [default_db, "", "nw", "sqlite:///nw.sqlite"]:
-        abs_db_url = f'sqlite:///{abspath(get_api_logic_server_dir())}/project_prototype_nw/nw.sqlite'
+        # abs_db_url = f'sqlite:///{abspath(get_api_logic_server_dir())}/project_prototype_nw/nw.sqlite'
+        abs_db_url = f'sqlite:///{abspath(get_api_logic_server_dir())}/project_prototype_nw/nw-gold-plus.sqlite'
         nw_db_status = "nw"
     elif db_url == "nw-":
         abs_db_url = f'sqlite:///{abspath(get_api_logic_server_dir())}/project_prototype_nw/nw.sqlite'
@@ -818,9 +862,10 @@ def about(ctx):
     """
         Recent Changes, system information.
     """
-    # print_info()
+    global recent_changes
+
     print(f'\tInstalled at {abspath(__file__)}\n')
-    print(f'\thttps://github.com/valhuber/ApiLogicServer/wiki/Tutorial\n')
+    print(f'\thttps://github.com/valhuber/ApiLogicServer/blob/main/README.md#api-logic-server---sample-tutorial\n')
 
     def print_at(label: str, value: str):
         tab_to = 30 - len(label)
@@ -842,47 +887,22 @@ def about(ctx):
     print_at('on hostname', hostname)
     print_at('cwd', os. getcwd())
     print_at("Python version", create_utils.run_command(f'python --version', msg="no-msg"))
+    print_at("Docker", is_docker())
 
     click.echo(
-        click.style(
-            f'\n\nRecent Changes:\n'
-            "\t11/26/2021 - 03.50.19: default search for favorite attr, indicate on label, yaml url fix  \n"
-            "\t11/25/2021 - 03.50.17: working search support in Config.js, admin chooser  \n"
-            "\t11/25/2021 - 03.50.16: patch to work with WSL as workaround for https://bugs.python.org/issue38633  \n"
-            "\t11/25/2021 - 03.50.15: folder rename - safrs-react-admin-npm-build, url for starting browser  \n"
-            "\t11/24/2021 - 03.50.14: constraint retry, pathlib fix  \n"
-            "\t11/23/2021 - 03.50.12: fix admin - resource type bug.  Parent & child joins, admin-attr  \n"
-            "\t11/22/2021 - 03.50.11: fixed config.js, child grids work.  No parent joins  \n"
-            "\t11/22/2021 - 03.50.10: upd build, web page default now admin (not swagger); detail grids still blank  \n"
-            "\t11/21/2021 - 03.50.09: web page default now admin (not swagger); detail grids still blank  \n"
-            "\t11/20/2021 - 03.50.08: safrs-admn join included, use table-name as resource-name; detail grids blank \n"
-            "\t11/20/2021 - 03.50.07: admin.yaml loaded/used by admin app, runs directly *and* ApiLogicServer run \n"
-            "\t11/19/2021 - 03.50.06: admin.yaml loaded/accepted by admin app, runs \n"
-            "\t11/18/2021 - 03.50.05: creates runnable admin-col.yaml, doc & console log info \n"
-            "\t11/16/2021 - 03.50.03: safrs-react-admin fixes \n"
-            "\t11/15/2021 - 03.50.02: minor diagnostics \n"
-            "\t11/13/2021 - 03.50.01: port conflict warning (not failure) for Docker \n"
-            "\t11/13/2021 - 03.50.00: rebuild-from-database/model, improved relationship support \n"
-            "\t11/04/2021 - 03.40.01: Per MacOS Monterey, default ports to 5001, 5002 \n"
-            "\t10/18/2021 - 03.20.11: Readme Tutorial for IDE users \n"
-            "\t10/16/2021 - 03.20.07: dev-network no longer required (reduce errors) \n"
-            "\t09/29/2021 - 03.01.15: run (now just runs without create), added create-and-run \n"
-            "\t09/25/2021 - 03.01.10: run command for Docker, pyodbc, fab create-by-copy, localhost swagger \n"
-            "\t09/15/2021 - 03.00.09: auto-create .devcontainer for vscode, configure network, python & debug \n"
-            "\t09/10/2021 - 03.00.02: rename logic_bank to declare_logic, improved logging\n"
-        )
+        click.style(recent_changes)
     )
 
 
 @main.command("create")
-@click.option('--db_url',
-              default=f'{default_db}',
-              prompt="SQLAlchemy Database URI",
-              help="SQLAlchemy Database URL - see above\n")
 @click.option('--project_name',
               default=f'{default_project_name}',
               prompt="Project to create",
               help="Create new directory here")
+@click.option('--db_url',
+              default=f'{default_db}',
+              prompt="SQLAlchemy Database URI",
+              help="SQLAlchemy Database URL - see above\n")
 @click.option('--from_git',
               default="",
               help="Template clone-from project (or directory)")
@@ -899,7 +919,7 @@ def about(ctx):
               default=True, is_flag=True,
               help="Creates ui/react app (yaml model)")
 @click.option('--flask_appbuilder/--no_flask_appbuilder',
-              default=True, is_flag=True,
+              default=False, is_flag=True,
               help="Creates ui/basic_web_app")
 @click.option('--react_admin/--no_react_admin',
               default=False, is_flag=True,
@@ -941,7 +961,6 @@ def create(ctx, project_name: str, db_url: str, not_exposed: str,
     """
     global command
     command = "create"
-    # print_info()
     db_types = ""
     api_logic_server(project_name=project_name, db_url=db_url,
                      not_exposed=not_exposed,
@@ -953,14 +972,14 @@ def create(ctx, project_name: str, db_url: str, not_exposed: str,
 
 
 @main.command("create-and-run")
+@click.option('--project_name',
+              default=f'{default_project_name}',
+              prompt="Project to create",
+              help="Create new directory here")
 @click.option('--db_url',
               default=f'{default_db}',
               prompt="SQLAlchemy Database URI",
               help="SQLAlchemy Database URL - see above\n")
-@click.option('--project_name',
-              default=f'{default_project_name}',
-              prompt="Project to create and run",
-              help="Create new directory here")
 @click.option('--from_git',
               default="",
               help="Template clone-from project (or directory)")
@@ -977,7 +996,7 @@ def create(ctx, project_name: str, db_url: str, not_exposed: str,
               default=True, is_flag=True,
               help="Creates ui/react app (yaml model)")
 @click.option('--flask_appbuilder/--no_flask_appbuilder',
-              default=True, is_flag=True,
+              default=False, is_flag=True,
               help="Creates ui/basic_web_app")
 @click.option('--react_admin/--no_react_admin',
               default=False, is_flag=True,
@@ -1019,7 +1038,6 @@ def create_and_run(ctx, project_name: str, db_url: str, not_exposed: str,
     """
     global command
     command = "create-and-run"
-    # print_info()
     db_types = ""
     api_logic_server(project_name=project_name, db_url=db_url,
                      not_exposed=not_exposed,
@@ -1031,14 +1049,14 @@ def create_and_run(ctx, project_name: str, db_url: str, not_exposed: str,
 
 
 @main.command("rebuild-from-database")
-@click.option('--db_url',
-              default=f'{default_db}',
-              prompt="SQLAlchemy Database URI",
-              help="SQLAlchemy Database URL - see above\n")
 @click.option('--project_name',
               default=f'{default_project_name}',
               prompt="Project to create",
               help="Create new directory here")
+@click.option('--db_url',
+              default=f'{default_db}',
+              prompt="SQLAlchemy Database URI",
+              help="SQLAlchemy Database URL - see above\n")
 @click.option('--from_git',
               default="",
               help="Template clone-from project (or directory)")
@@ -1055,7 +1073,7 @@ def create_and_run(ctx, project_name: str, db_url: str, not_exposed: str,
               default=True, is_flag=True,
               help="Creates ui/react app (yaml model)")
 @click.option('--flask_appbuilder/--no_flask_appbuilder',
-              default=True, is_flag=True,
+              default=False, is_flag=True,
               help="Creates ui/basic_web_app")
 @click.option('--react_admin/--no_react_admin',
               default=False, is_flag=True,
@@ -1098,12 +1116,11 @@ def rebuild_from_database(ctx, project_name: str, db_url: str, not_exposed: str,
 \b
         ex
 \b
-        ApiLogicServer rebuild-from-database --project_name=~/dev/servers/api_logic_server --db_url=nw
+        ApiLogicServer rebuild-from-database --project_name=~/dev/servers/ApiLogicProject --db_url=nw
 
     """
     global command
     command = "rebuild-from-database"
-    # print_info()
     db_types = ""
     api_logic_server(project_name=project_name, db_url=db_url,
                      not_exposed=not_exposed,
@@ -1115,14 +1132,14 @@ def rebuild_from_database(ctx, project_name: str, db_url: str, not_exposed: str,
 
 
 @main.command("rebuild-from-model")
-@click.option('--db_url',
-              default=f'{default_db}',
-              prompt="SQLAlchemy Database URI",
-              help="SQLAlchemy Database URL - see above\n")
 @click.option('--project_name',
               default=f'{default_project_name}',
               prompt="Project to create",
               help="Create new directory here")
+@click.option('--db_url',
+              default=f'{default_db}',
+              prompt="SQLAlchemy Database URI",
+              help="SQLAlchemy Database URL - see above\n")
 @click.option('--from_git',
               default="",
               help="Template clone-from project (or directory)")
@@ -1139,7 +1156,7 @@ def rebuild_from_database(ctx, project_name: str, db_url: str, not_exposed: str,
               default=True, is_flag=True,
               help="Creates ui/react app (yaml model)")
 @click.option('--flask_appbuilder/--no_flask_appbuilder',
-              default=True, is_flag=True,
+              default=False, is_flag=True,
               help="Creates ui/basic_web_app")
 @click.option('--react_admin/--no_react_admin',
               default=False, is_flag=True,
@@ -1181,7 +1198,6 @@ def rebuild_from_model(ctx, project_name: str, db_url: str, not_exposed: str,
     """
     global command
     command = "rebuild-from-model"
-    # print_info()
     db_types = ""
     api_logic_server(project_name=project_name, db_url=db_url,
                      not_exposed=not_exposed,
@@ -1213,7 +1229,7 @@ def run_api(ctx, project_name: str, host: str="localhost", port: str="5656"):
         Example
 
 \b
-            ApiLogicServer run --project_name=/localhost/api_logic_server
+            ApiLogicServer run --project_name=/localhost/ApiLogicProject
             ApiLogicServer run --project_name=    # runs last-created project
     """
     global command
@@ -1296,7 +1312,7 @@ def run_ui(ctx, project_name: str, host: str="localhost", port: str="5002"):
         Example
 
 \b
-            ApiLogicServer run-ui --project_name=/localhost/api_logic_server
+            ApiLogicServer run-ui --project_name=/localhost/ApiLogicProject
             ApiLogicServer run-ui --project_name=    # runs last-created project
     """
     global command
@@ -1316,60 +1332,21 @@ def examples(ctx):
     """
     Example commands, including SQLAlchemy URIs.
     """
-    info = [
-        'Examples:',
-        '  ApiLogicServer create-and-run',
-        '  ApiLogicServer create-and-run --db_url=sqlite:///nw.sqlite',
-        '  ApiLogicServer create-and-run --db_url=mysql+pymysql://root:p@mysql-container:3306/classicmodels '
-        '--project_name=/localhost/docker_db_project',
-        '  ApiLogicServer create-and-run --db_url=mssql+pyodbc://sa:posey386!@localhost:1433/NORTHWND?'
-        'driver=ODBC+Driver+17+for+SQL+Server?trusted_connection=no',
-        '  ApiLogicServer create-and-run --db_url=postgresql://postgres:p@10.0.0.234/postgres',
-        '  ApiLogicServer create-and-run --db_url=postgresql+psycopg2:'
-        '//postgres:password@localhost:5432/postgres?options=-csearch_path%3Dmy_db_schema',
-        '  ApiLogicServer create --host=ApiLogicServer.pythonanywhere.com --port=',
-        '',
-        'Where --db_url defaults to supplied sample, or, specify URI for your own database:',
-        '   SQLAlchemy Database URI help: https://docs.sqlalchemy.org/en/14/core/engines.html',
-        '   Other URI examples:           https://github.com/valhuber/ApiLogicServer/wiki/Testing',
-        ' ',
-        'Docs: https://github.com/valhuber/ApiLogicServer#readme\n'
-    ]
-    for each_line in info:
-        print(each_line)
+    print_uri_info()
+
 
 log = logging.getLogger(__name__)
 
 
-def print_info():
+def print_uri_info():
     """
     Creates and optionally runs a customizable ApiLogicServer project, Example
 
     URI examples, Docs URL
     """
-    info = [
-        '',
-        'Creates and optionally runs a customizable ApiLogicServer project',
-        '',
-        'Examples:',
-        '  ApiLogicServer create-and-run',
-        '  ApiLogicServer create-and-run --db_url=sqlite:///nw.sqlite',
-        '  ApiLogicServer create-and-run --db_url=mysql+pymysql://root:p@mysql-container:3306/classicmodels '
-        '--project_name=/localhost/docker_db_project',
-        '  ApiLogicServer create-and-run --db_url=mssql+pyodbc://sa:posey386!@localhost:1433/NORTHWND?'
-        'driver=ODBC+Driver+17+for+SQL+Server?trusted_connection=no',
-        '  ApiLogicServer create-and-run --db_url=postgresql://postgres:p@10.0.0.234/postgres',
-        '  ApiLogicServer create-and-run --db_url=postgresql+psycopg2:'
-        '//postgres:password@localhost:5432/postgres?options=-csearch_path%3Dmy_db_schema',
-        '  ApiLogicServer create --host=ApiLogicServer.pythonanywhere.com --port=',
-        '',
-        'Where --db_url defaults to supplied sample, or, specify URI for your own database:',
-        '   SQLAlchemy Database URI help: https://docs.sqlalchemy.org/en/14/core/engines.html',
-        '   Other URI examples:           https://github.com/valhuber/ApiLogicServer/wiki/Testing',
-        ' ',
-        'Docs: https://github.com/valhuber/ApiLogicServer#readme\n'
-    ]
-    for each_line in info:
+
+    global uri_info
+    for each_line in uri_info:
         sys.stdout.write(each_line + '\n')
     sys.stdout.write('\n')
 
@@ -1417,7 +1394,7 @@ def start():               # target of setup.py
 command = "not set"
 if __name__ == '__main__':  # debugger & python command line start here
     # eg: python api_logic_server_cli/cli.py create --project_name=~/Desktop/test_project
-    # unix: python api_logic_server_cli/cli.py create --project_name=/home/api_logic_server
+    # unix: python api_logic_server_cli/cli.py create --project_name=/home/ApiLogicProject
 
     print(f'\nWelcome to API Logic Server, {__version__}\n')  #  at {local_ip} ')
     hostname, local_ip = check_ports()
