@@ -7,7 +7,7 @@ from os.path import abspath
 import importlib.util
 import sys
 import os
-from typing import NewType
+from typing import NewType, Type
 import sqlalchemy
 import sqlalchemy.ext
 from sqlalchemy import MetaData
@@ -33,15 +33,39 @@ MetaDataTable = NewType('MetaDataTable', object)
 
 
 class ResourceAttribute():
-    def __init__(self, name):
+    def __init__(self, name: str, resource: Type['Resource'], type: str):
         self.name = name
+        self.type = None  # none means not interesting, default display to simple text
+        if name == "InvoiceDate":
+            debug_str = "Nice breakpoint"
+        if type == "DECIMAL":
+            self.type = "DECIMAL"
+        elif type == "DATE":
+            self.type = "DATE"
+#        elif type == "DATETIME":  safrs-admin date-picker fails with bad datetime format
+#            self.type = "DATE"
+        elif type == "IMAGE":
+            self.type = "IMAGE"
+        elif type.startswith("NTEXT") == "NTEXT":
+            self.type = "NTEXT"
+        self.non_favorite = False
+        lower_name = name.lower()
+        non_favs = resource.create_from_model._non_favorite_names_list
+        for each_non_fav in non_favs:
+            if lower_name.endswith(each_non_fav):
+                self.non_favorite = True
+                break
+        resource.attributes.append(self)
 
     def __str__(self):
-        return self.name
+        result = self.name
+        if self.type is not None:
+            result += " - " + self.type
+        return result
 
 
 class ResourceRelationship():
-    def __init__(self, parent_role_name, child_role_name):
+    def __init__(self, parent_role_name: str, child_role_name: str):
         self.parent_role_name = parent_role_name
         self.child_role_name = child_role_name
         self.parent_resource = None
@@ -57,17 +81,44 @@ class ResourceRelationship():
 
 
 class Resource():
-    def __init__(self, name):
+    def __init__(self, name: str, create_from_model):
         self.name = name        # class name (which != safrs resource name)
         self.table_name = name  # safrs resource name; this is just default, overridden in create_model
         self.type = name        # just default, overridden in create_model
         self.children: List[ResourceRelationship] = list()
         self.parents: List[ResourceRelationship] = list()
         self.attributes: List[ResourceAttribute] = list()
+        self.create_from_model = create_from_model  # to find favorite names etc.
 
     def __str__(self):
         return f'Resource: {self.name}, table_name: {self.table_name}, type: {self.type}'
 
+    def get_favorite_attribute(self) -> ResourceAttribute:
+        """
+             returns ResourceAttribute of first attribute that is...
+                 named <favorite_name> (default to "name"), else
+                 containing <favorite_name>, else
+                 (or first column)
+
+             Parameters
+                 argument1 a_table_name - str
+
+             Returns
+                 string of column name that is favorite (e.g., first in list)
+         """
+        favorite_names = self.create_from_model._favorite_names_list
+        for each_favorite_name in favorite_names:
+            for each_attribute in self.attributes:
+                attribute_name = each_attribute.name.lower()
+                if attribute_name == each_favorite_name:
+                    return each_attribute
+            for each_attribute in self.attributes:
+                attribute_name = each_attribute.name.lower()
+                if each_favorite_name in attribute_name:
+                    return each_attribute
+        for each_attribute in self.attributes:  # no favorites, just return 1st
+            return each_attribute
+
 
 class CreateFromModel(object):
     """
@@ -432,7 +483,7 @@ class CreateFromModel(object):
         """
         result = list()
         if a_resource.name == "Order":  # for debug
-            log.debug("predictive_joins for: " + a_resource.name)
+            debug_str = "predictive_joins for: " + a_resource.name
         for each_parent in a_resource.parents:
             each_parent_resource = self.resource_list[each_parent.parent_resource]
             favorite_attribute_name = self.favorite_attribute_name(each_parent_resource)
@@ -455,7 +506,7 @@ class CreateFromModel(object):
         result = list()
         foreign_keys = a_table_def.foreign_key_constraints
         if a_table_def.name == "Order":  # for debug
-            log.debug("predictive_joins for: " + a_table_def.name)
+            debug_str = "predictive_joins for: " + a_table_def.name
         for each_foreign_key in foreign_keys:
             """ remove old code
             each_parent_name = each_foreign_key.referred_table.name + "." + each_foreign_key.column_keys[0]
@@ -511,7 +562,7 @@ class CreateFromModel(object):
             parents = each_possible_child.foreign_keys
             if (a_table_def.name == "Customer" and
                     each_possible_child.name == "Order"):
-                log.debug(a_table_def)
+                debug_str = a_table_def
             for each_parent in parents:
                 each_parent_name = each_parent.target_fullname
                 loc_dot = each_parent_name.index(".")
@@ -608,7 +659,7 @@ class CreateFromModel(object):
         if table_name in self.table_to_class_map:
             return self.table_to_class_map[table_name]
         else:
-            log.debug("skipping view: " + table_name)
+            debug_str = "skipping view: " + table_name
             return None
 
     def find_meta_data(self, cwd: str, log_info: bool=False) -> MetaData:
@@ -669,7 +720,9 @@ class CreateFromModel(object):
                         resource_name = each_cls_member[0]
                         resource_class = each_cls_member[1]
                         table_name = resource_class._s_collection_name
-                        resource = Resource(name=resource_name)
+                        if table_name == "Department":
+                            debug_str = "Excellent breakpoint"
+                        resource = Resource(name=resource_name, create_from_model=self)
                         self.metadata = resource_class.metadata
                         self.table_to_class_map.update({table_name: resource_name})   # required for ui_basic_web_app
                         if resource_name not in resource_list:
@@ -679,15 +732,33 @@ class CreateFromModel(object):
                         resource_data = {"type": resource_class._s_type}  # todo what's this?
                         resource_data = {"type": resource_name}
                         for each_attribute in resource_class._s_columns:
-                            resource_attribute = ResourceAttribute(name=str(each_attribute.name))
-                            resource.attributes.append(resource_attribute)
+                            attr_type = str(each_attribute.type)
+                            resource_attribute = ResourceAttribute(name=str(each_attribute.name),
+                                                                   resource=resource,
+                                                                   type=attr_type)
                         for rel_name, rel in resource_class._s_relationships.items():
-                            relation = {}
-                            relation["direction"] = "toone" if rel.direction == MANYTOONE else "tomany"
-                            if rel.direction == MANYTOONE:  # process only parents of this child
-                                relationship = ResourceRelationship(rel_name, rel.backref)
+                            # relation = {}
+                            # relation["direction"] = "toone" if rel.direction == MANYTOONE else "tomany"
+                            if rel.direction == ONETOMANY:  # process only parents of this child
+                                debug_str = "onetomany"
+                            else:  # many to one (parent for this child) - version <= 3.50.43
+                                debug_rel = False
+                                if debug_rel:
+                                    debug_rel_str = f'Debug resource_class._s_relationships {resource_name}: ' \
+                                                    f'parent_role_name (aka rel_name): {rel_name},    ' \
+                                                    f'child_role_name (aka rel.back_populates): {rel.back_populates}'
+                                    print(debug_rel_str)
+
+                                parent_role_name = rel_name
+                                child_role_name = rel.back_populates
+                                do_patch_self_reln = False  # roles backward for self-relns, but addressed in codegen
+                                if do_patch_self_reln and resource_name == rel.mapper.class_._s_class_name:
+                                    parent_role_name = rel.back_populates
+                                    child_role_name = rel_name
+                                relationship = ResourceRelationship(parent_role_name=parent_role_name,
+                                                                    child_role_name=child_role_name)
                                 for each_fkey in rel._calculated_foreign_keys:
-                                    pair = ( "?", each_fkey.description)
+                                    pair = ("?", each_fkey.description)
                                     relationship.parent_child_key_pairs.append(pair)
                                 resource.parents.append(relationship)
                                 relationship.child_resource = resource_name
@@ -695,14 +766,14 @@ class CreateFromModel(object):
                                 parent_resource_name = rel.mapper.class_._s_class_name
                                 relationship.parent_resource = parent_resource_name
                                 if parent_resource_name not in resource_list:
-                                    parent_resource = Resource(name=parent_resource_name)
+                                    parent_resource = Resource(name=parent_resource_name, create_from_model=self)
                                     resource_list[parent_resource_name] = parent_resource
                                 parent_resource = resource_list[parent_resource_name]
                                 parent_resource.children.append(relationship)
                     pass
                 pass
-                log.debug(f'setting resource_list: {str(resource_list)}')
-                self.resource_list = resource_list  # currently, you can disable this to bypass errors
+                debug_str = f'setting resource_list: {str(resource_list)}'
+                self.resource_list = resource_list  # model loaded - excellent breakpoint location
 
                 if orm_class is not None:
                     log.debug(f'.. .. ..Dynamic model import successful '
