@@ -2,6 +2,7 @@ import logging
 import shutil
 import sys
 import os
+import pathlib
 from pathlib import Path
 from typing import NewType, List
 
@@ -23,10 +24,10 @@ log.addHandler(handler)
 log.propagate = True
 
 # temp hacks for admin app migration to attributes
-admin_col_is_active = True
-admin_col_is_join_active = False
-admin_col_tab_columns = False
-admin_user_key_is_active = True  # resource property, in lieu of admin_col_tab_columns
+
+admin_attr_ordering = True
+admin_parent_joins_implicit = True  # True => id's displayed as joins, False => explicit parent join attrs
+admin_child_grids = False  # True => identify each child grid attr explicitly, False => use main grid definition
 admin_relationships_with_parents = True
 
 # have to monkey patch to work with WSL as workaround for https://bugs.python.org/issue38633
@@ -44,7 +45,6 @@ def patched_copyxattr(src, dst, *, follow_symlinks=True):
 shutil._copyxattr = patched_copyxattr
 
 
-
 #  MetaData = NewType('MetaData', object)
 MetaDataTable = NewType('MetaDataTable', object)
 
@@ -93,8 +93,7 @@ class AdminCreator(object):
         self.connection = None
         self.app = None
         self.admin_yaml = DotMap()
-        self.admin_yaml_col = DotMap()
-        self.max_list_columns = 7  # maybe make this a param
+        self.max_list_columns = 8  # maybe make this a param
 
         self._non_favorite_names_list = self.non_favorite_names.split()
         self._favorite_names_list = self.favorite_names.split()
@@ -111,26 +110,26 @@ class AdminCreator(object):
         cwd = os.getcwd()
         sys.path.append(cwd)
 
-        self.admin_yaml.api_root = "http://localhost:5656/api"
-        self.admin_yaml_col.api_root = "http://localhost:5656/api"
+        self.admin_yaml.api_root = f'http://localhost:5656/api'
+        if self.host != "localhost":
+            if self.port !="":
+                self.admin_yaml.api_root = f'http://{self.host}:{self.port}/api'
+            else:
+                self.admin_yaml.api_root = f'http://{self.host}/api'
         self.admin_yaml.resources = {}
         for each_resource_name in self.mod_gen.resource_list:
             each_resource = self.mod_gen.resource_list[each_resource_name]
             self.create_resource_in_admin(each_resource)
-            if admin_col_is_active:
-                self.create_resource_in_admin_col(each_resource)
 
         self.create_about()
         self.create_info()
         self.create_settings()
-        self.doc_properties()
+        # self.doc_properties()
 
         admin_yaml_dict = self.admin_yaml.toDict()
         admin_yaml_dump = yaml.dump(admin_yaml_dict)
-        admin_yaml_dict_col = self.admin_yaml_col.toDict()
-        admin_yaml_dump_col = yaml.dump(admin_yaml_dict_col)
         if self.mod_gen.command != "create-ui":
-            self.write_yaml_files(admin_yaml_dump, admin_yaml_dump_col)
+            self.write_yaml_files(admin_yaml_dump)
         return admin_yaml_dump
 
     def create_resource_in_admin(self, resource: Resource):
@@ -141,8 +140,7 @@ class AdminCreator(object):
             new_resource = DotMap()
             self.num_pages_generated += 1
             new_resource.type = str(resource.name)
-            if admin_user_key_is_active:
-                new_resource.user_key = str(self.mod_gen.favorite_attribute_name(resource))
+            new_resource.user_key = str(self.mod_gen.favorite_attribute_name(resource))
 
             self.create_attributes_in_owner(new_resource, resource, None)
             child_tabs = self.create_child_tabs(resource)
@@ -151,97 +149,155 @@ class AdminCreator(object):
             self.admin_yaml.resources[resource.table_name] = new_resource.toDict()
 
     def create_attributes_in_owner(self, owner: DotMap, resource: Resource, owner_resource: (None, Resource)):
-        """ create attribute in owner (DotMap - of resource or tab)
+        """ create attributes in owner (owner is a DotMap -- of resource, or tab)
+
+            Order:
+                attributes:  1 Favorite,  2 Joins,  3 Others / not favs,  4 Not Favs
+                - label: ShipName*
+                  name: ShipName
+                  search: true
+                - name: OrderDate
+                - name: RequiredDate
+                - name: Id
+                - name: CustomerId
+            """
+        owner.attributes = []
+        attributes_dict = []  # DotMap()
+        processed_attributes = set()
+
+        # Step 1 - favorite attribute
+        favorite_attribute = resource.get_favorite_attribute()
+        admin_attribute = self.create_admin_attribute(favorite_attribute)
+        if admin_attribute is None:
+            favorite_attribute = resource.attributes[0]
+            admin_attribute = self.create_admin_attribute(favorite_attribute)
+        processed_attributes.add(favorite_attribute.name)
+        admin_attribute.search = True
+        admin_attribute.sort = True
+        admin_attribute.label = f"{self.cap_space(favorite_attribute.name)}*"
+        attributes_dict.append(admin_attribute)
+
+        # Step 2 - Parent Joins
+        for each_parent in resource.parents:
+            if admin_parent_joins_implicit:  # temp hack - just do the FK
+                fk_pair = each_parent.parent_child_key_pairs[0]  # assume single-field keys
+                fk_attr_name = fk_pair[1]
+                processed_attributes.add(fk_attr_name)
+                admin_attribute = self.create_admin_attribute(fk_attr_name)
+                if admin_attribute is not None:
+                    attributes_dict.append(admin_attribute)
+            else:
+                pass
+                """  perhaps something like this:
+                      - Location:     <— this is the parent resource name
+                          fks:
+                          - City       <- child FKs
+                          - Country
+                          attributes:  <- parent attrs to display
+                          - name: city
+                          - name: country
+
+                """
+
+        # Step 3 - Other fields, except non-favorites
+        for each_attribute in resource.attributes:
+            if each_attribute.name not in processed_attributes:
+                if not each_attribute.non_favorite:
+                    processed_attributes.add(each_attribute.name)
+                    admin_attribute = self.create_admin_attribute(each_attribute)
+                    if admin_attribute is not None:
+                        attributes_dict.append(admin_attribute)
+
+        # Step 4 - Non-favorites
+        for each_attribute in resource.attributes:
+            if each_attribute.name not in processed_attributes:
+                if each_attribute.non_favorite:
+                    processed_attributes.add(each_attribute.name)
+                    admin_attribute = self.create_admin_attribute(each_attribute)
+                    if admin_attribute is not None:
+                        attributes_dict.append(admin_attribute)
+
+        owner.attributes = attributes_dict
+
+    @staticmethod
+    def create_admin_attribute(resource_attribute) -> DotMap:
+        attribute_name = resource_attribute if isinstance(resource_attribute, str) else resource_attribute.name
+        admin_attribute = DotMap()
+        admin_attribute.name = str(attribute_name)
+        if attribute_name == "InvoiceDate":
+            log.debug("Good breakpoint location")
+        if not isinstance(resource_attribute, str):
+            if resource_attribute.type in ["DECIMAL", "DATE"]:
+                admin_attribute.type = resource_attribute.type
+            if resource_attribute.type in ["NTEXT", "IMAGE"]:
+                admin_attribute = None
+        return admin_attribute
+
+    @staticmethod
+    def cap_space(text):
+        new_text = ' '
+        for i, letter in enumerate(text):
+            if i and letter.isupper():
+                new_text += ' '
+            new_text += letter
+        return new_text
+
+    def create_attributes_in_owner_zz(self, owner: DotMap, resource: Resource, owner_resource: (None, Resource)):
+        """ create attributes in owner (owner is a DotMap -- of resource, or tab)
 
           Customer:
             attributes:
-            - CompanyName
-            - ContactName
+            - label: CompanyName*
+              name: CompanyName
+              search: true
+            - name: ContactName
+            - name: ContactTitle
         """
         owner.attributes = []
-        attributes = set()
-        if admin_col_tab_columns:
+        attributes_dict = []  # DotMap()
+        if admin_attr_ordering:
             attributes = self.mod_gen.get_show_attributes(resource)
         else:
             attributes = self.mod_gen.get_attributes(resource)
         for each_attribute in attributes:
-            if "." not in each_attribute:
+            if "." not in each_attribute:   # not a parent join
+                admin_attribute = DotMap()
+                admin_attribute.name = each_attribute
                 if each_attribute == self.mod_gen.favorite_attribute_name(resource):
-                    attribute_with_search = DotMap()
-                    search = DotMap()
-                    search.search = True
-                    search.label = f'{each_attribute}*'  # adding space causes newline, so omit for now
-                    attribute_with_search[each_attribute] = search
-                    owner.attributes.append(attribute_with_search)
-                else:
-                    owner.attributes.append(each_attribute)
-            else:
+                    admin_attribute.search = True
+                    admin_attribute.label = f"{each_attribute}*"
+                """
+                if each_attribute.type in ["DECIMAL", "DATE"]:
+                    admin_attribute.type = each_attribute.type
+                """
+                attributes_dict.append(admin_attribute)
+            else:                           # parent join (disabled code - overwritten at end)
                 relationship = self.new_relationship_to_parent(resource, each_attribute, owner_resource)
                 if relationship is not None:  # skip redundant master join
                     rel = DotMap()
                     parent_role_name = each_attribute.split('.')[0]
                     rel[parent_role_name] = relationship.toDict()
                     owner.attributes.append(rel)
-
-    def create_resource_in_admin_col(self, resource: Resource):
-        """ self.admin_yaml.resources += resource DotMap for given resource
-        """
-        resource_name = resource.name
-        if self.do_process_resource(resource_name):
-            new_resource = DotMap()
-            self.num_pages_generated += 1
-            new_resource.type = str(resource.name)
-            new_resource.label = resource.name + " - label"
-            if admin_user_key_is_active:
-                new_resource.user_key = str(self.mod_gen.favorite_attribute_name(resource))
-
-            self.create_columns_in_owner(new_resource, resource, None)
-            child_tabs = self.create_child_tabs_col(resource)
-            new_resource.relationships = child_tabs
-
-            self.admin_yaml_col.resources[resource_name] = new_resource.toDict()
-
-    def create_columns_in_owner(self, owner: DotMap, resource: Resource, owner_resource: (None, Resource)):
-        """ create columns in owner (DotMap - of resource or tab)
-        """
-        owner.columns = []
-        attributes = set()
-        if admin_col_tab_columns:
-            attributes = self.mod_gen.get_show_attributes(resource)
-        else:
-            attributes = self.mod_gen.get_attributes(resource)
-        for each_attribute in attributes:
-            if "." not in each_attribute:
-                col_def = DotMap()
-                col_def.name = each_attribute
-                owner.columns.append(col_def)
-            elif admin_col_is_join_active:
-                relationship = self.new_relationship_to_parent(resource, each_attribute, owner_resource)
-                if relationship is not None:  # skip redundant master join
-                    rel = DotMap()
-                    parent_role_name = each_attribute.split('.')[0]
-                    rel[parent_role_name] = relationship.toDict()
-                    owner.columns.append(rel)
-            else:
-                log.debug(f'column skipped since admin_col_is_join_active is false: '
-                          f'{resource.name}.{each_attribute} ')
+        owner.attributes = attributes_dict  # for now, just attrs, no parent joins since safrs-react autojoins FKs
 
     def new_relationship_to_parent(self, a_child_resource: Resource, parent_attribute_reference,
-                                   a_master_parent_resource) -> (None, DotMap):
+                                      a_master_parent_resource) -> (None, DotMap):
         """
         given a_child_table_def.parent_column_reference, create relationship: attrs, fKeys (for *js* client (no meta))
 
           Order:
             attributes:
             - ShipName
+            - Amount
             - Location:
                 fks:
                 - City
                 - Country
-                show_attributes: []
-                type: Location
+                attributes:
+                - name: city
+                - name: country
 
-        :param a_child_resource: a child table (not class), eg, Employees
+        :param a_child_resource: a child resource (not class), eg, Employees
         :param parent_attribute_reference: parent ref, eg, Department1.DepartmentName
         :param a_master_parent_resource: the master of master/detail - skip joins for this
         """
@@ -266,10 +322,10 @@ class AdminCreator(object):
             if parent_role_name not in self.multi_reln_exceptions:
                 self.multi_reln_exceptions.append(parent_role_name)
                 log.warning(f'Error - please search ui/admin/admin.yaml for: Unable to find role')
-        relationship.type = str(parent_relationship.parent_resource)
-        relationship.show_attributes = []
+        relationship.resource = str(parent_relationship.parent_resource)  # redundant??
+        relationship.attributes = []
         relationship.fks = []
-        if a_child_resource.name == "Employee":
+        if a_child_resource.name == "Order":
             log.debug("Parents for special table - debug")
         for each_column in parent_relationship.parent_child_key_pairs:  # XXX FIXME
             # key_column = DotMap()
@@ -278,25 +334,24 @@ class AdminCreator(object):
         # todo - verify fullname is table name (e.g, multiple relns - emp.worksFor/onLoan)
         return relationship
 
-    def create_child_tabs(self, resource: Resource) -> DotMap:
+    def create_child_tabs(self, resource: Resource) -> List:
         """
         build tabs for related children
 
-            tab_groups:
-              CustomerCustomerDemoList:
-                attributes:
-                - Id
-                - CustomerTypeId
-                fkeys:
-                  source_delete_me: '?'
-                  target: CustomerTypeId
-                resource: CustomerCustomerDemo
+        tab_groups:
+          CustomerCustomerDemoList:
+            direction: tomany
+            fks:
+            - CustomerTypeId
+            resource: CustomerCustomerDemo
         """
         if len(self.mod_gen.resource_list) == 0:   # almost always, use_model false (we create)
             return self.create_child_tabs_no_model(resource)
 
+        if resource.name == "Department":  # excellent breakpoint location
+            log.debug(f'Relationships for {resource.name}')
         children_seen = set()
-        tab_group = DotMap()
+        tab_group = []
         for each_resource_relationship in resource.children:
             each_resource_tab = DotMap()
             self.num_related += 1
@@ -306,79 +361,27 @@ class AdminCreator(object):
             children_seen.add(each_child)
             each_resource_tab.fks = []
             for each_pair in each_resource_relationship.parent_child_key_pairs:
-                each_resource_tab.fks.append(each_pair[1])
-                """
-                key_pair = DotMap()
-                key_pair.target = each_pair[1]
-                key_pair.source_delete_me = each_pair[0]
-                each_resource_tab.fks = key_pair
-                """
-
-            each_resource_tab.resource = str(each_child)
-            each_resource_tab.direction = "tomany"
-            tab_name = each_resource_relationship.child_role_name
-
+                each_resource_tab.fks.append(str(each_pair[1]))
             each_child_resource = self.mod_gen.resource_list[each_child]
-            if admin_col_tab_columns:
-                self.create_attributes_in_owner(each_resource_tab, each_child_resource, resource)
-            tab_group[tab_name] = each_resource_tab  # disambiguate multi-relns, eg Employee OnLoan/WorksForDept
-        if admin_relationships_with_parents:
-            for each_resource_relationship in resource.parents:
-                each_resource_tab = DotMap()
-                each_resource_tab.name = each_resource_relationship.parent_role_name
-                each_parent = each_resource_relationship.parent_resource
-                each_resource_tab.target = str(each_parent)
-                each_resource_tab.direction = "toone"
-                each_resource_tab.fks = []
-                for each_pair in each_resource_relationship.parent_child_key_pairs:
-                    each_resource_tab.fks.append(each_pair[1])
-                tab_name = each_resource_relationship.parent_role_name
-
-                # tab_group[tab_name] = each_resource_tab  # disambiguate multi-relns, eg Employee OnLoan/WorksForDept
-                tab_group[tab_name] = each_resource_tab
-        return tab_group
-
-    def create_child_tabs_col(self, resource: Resource) -> DotMap:
-        """
-        build tabs for all related children, col style
-        """
-        if len(self.mod_gen.resource_list) == 0:   # almost always, use_model false (we create)
-            return self.create_child_tabs_no_model(resource)
-
-        children_seen = set()
-        tab_group = []  # ??  DotMap()
-        for each_resource_relationship in resource.children:
-            each_resource_tab = DotMap()
+            each_resource_tab.resource = each_child_resource.table_name
+            each_resource_tab.direction = "tomany"
             each_resource_tab.name = each_resource_relationship.child_role_name
-            self.num_related += 1
-            each_child = each_resource_relationship.child_resource
-            if each_child in children_seen:
-                pass  # it's ok, we are using the child_role_name now
-            each_resource_tab.target = str(each_child)
-            each_resource_tab.direction = "tomany"
-            children_seen.add(each_child)
-            each_resource_tab.fks = []
-            for each_pair in each_resource_relationship.parent_child_key_pairs:
-                each_resource_tab.fks.append(each_pair[1])
-            tab_name = each_resource_relationship.child_role_name
-
             each_child_resource = self.mod_gen.resource_list[each_child]
-            if admin_col_tab_columns:
-                self.create_columns_in_owner(each_resource_tab, each_child_resource, resource)
-
-            # tab_group[tab_name] = each_resource_tab  # disambiguate multi-relns, eg Employee OnLoan/WorksForDept
-            tab_group.append(each_resource_tab)
+            if admin_child_grids:
+                self.create_attributes_in_owner(each_resource_tab, each_child_resource, resource)
+            tab_group.append(each_resource_tab)  # disambiguate multi-relns, eg Employee OnLoan/WorksForDept
         if admin_relationships_with_parents:
             for each_resource_relationship in resource.parents:
                 each_resource_tab = DotMap()
-                each_resource_tab.name = each_resource_relationship.parent_role_name
                 each_parent = each_resource_relationship.parent_resource
-                each_resource_tab.target = str(each_parent)
+                each_resource_tab.resource = str(each_parent)
+                each_parent_resource = self.mod_gen.resource_list[each_parent]
+                each_resource_tab.resource = each_parent_resource.table_name
                 each_resource_tab.direction = "toone"
                 each_resource_tab.fks = []
                 for each_pair in each_resource_relationship.parent_child_key_pairs:
-                    each_resource_tab.fks.append(each_pair[1])
-                tab_name = each_resource_relationship.parent_role_name
+                    each_resource_tab.fks.append(str(each_pair[1]))
+                each_resource_tab.name = each_resource_relationship.parent_role_name
 
                 # tab_group[tab_name] = each_resource_tab  # disambiguate multi-relns, eg Employee OnLoan/WorksForDept
                 tab_group.append(each_resource_tab)
@@ -499,19 +502,18 @@ class AdminCreator(object):
         parent_path = parent_path.parent
         return parent_path
 
-    def write_yaml_files(self, admin_yaml, admin_yaml_col):
+    def write_yaml_files(self, admin_yaml):
         """ write admin.yaml, with backup, with additional nw customized backup
         """
         yaml_file_name = self.mod_gen.fix_win_path(self.mod_gen.project_directory + f'/ui/admin/admin.yaml')
+        """  todo - overwrite if never altered
+        import os.path, time
+        print("Last modified: %s" % time.ctime(os.path.getmtime("test.txt")))
+        print("Created: %s" % time.ctime(os.path.getctime("test.txt")))
+        """
         if not self.mod_gen.command.startswith("rebuild"):
             with open(yaml_file_name, 'w') as yaml_file:
                 yaml_file.write(admin_yaml)
-            if admin_col_is_active:
-                yaml_file_name_col = self.mod_gen.fix_win_path(
-                    self.mod_gen.project_directory + f'/ui/admin/admin-col.yaml')
-                print(f'.. .. .. ..Creating temp {yaml_file_name_col}')
-                with open(yaml_file_name_col, 'w') as yaml_file_col:
-                    yaml_file_col.write(admin_yaml_col)
 
         yaml_created_file_name = \
             self.mod_gen.fix_win_path(self.mod_gen.project_directory + f'/ui/admin/admin-created.yaml')
@@ -527,20 +529,23 @@ class AdminCreator(object):
             admin_file = open(nw_backup_file_name, 'w')
             admin_file.write(admin_custom_nw)
             admin_file.close()
-            dev_temp_do_not_overwrite = True  # fixme remove this when the files are stable
+            dev_temp_do_not_overwrite = False
             if not dev_temp_do_not_overwrite:
+                print('.. .. ..Using customized admin_custom_nw.yaml - compare to admin-created.yaml')
                 admin_file = open(yaml_file_name, 'w')
                 admin_file.write(admin_custom_nw)
                 admin_file.close()
 
-                nw_backup_file_name = yaml_file_name.replace("admin.yaml", "admin_custom_nw_backup.yaml")
-                admin_file = open(nw_backup_file_name, 'w')
-                admin_file.write(admin_custom_nw)
-                admin_file.close()
-
     def create_settings(self):
         self.admin_yaml.settings = DotMap()
-        self.admin_yaml.settings.max_list_columns = str(self.max_list_columns)
+        self.admin_yaml.settings.max_list_columns = self.max_list_columns
+        home_js = "http://localhost:5656/admin-app/home.js"
+        if self.host != "localhost":
+            if self.port !="":
+                home_js = f'http://{self.host}:{self.port}/admin-app/home.js'
+            else:
+                home_js = f'http://{self.host}/admin-app/home.js'
+        self.admin_yaml.settings.HomeJS = home_js
         return
 
     def create_about(self):
@@ -598,13 +603,25 @@ class AdminCreator(object):
         """
         from_proto_dir = from_git
         if from_proto_dir == "":
-            from_proto_dir = self.mod_gen.fix_win_path(str(self.get_create_from_model_dir()) +
-                                                       "/create_from_model/safrs-react-admin-npm-build")
-        to_project_dir = self.mod_gen.fix_win_path(self.mod_gen.project_directory + "/ui/admin")
+            from_proto_dir = pathlib.Path(self.get_create_from_model_dir()).\
+                joinpath("create_from_model", "safrs-react-admin-npm-build")
+        to_project_dir = pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "safrs-react-admin")
         print(f'{msg} copy prototype admin project {from_proto_dir} -> {to_project_dir}')
-        # self.mod_gen.recursive_overwrite(from_proto_dir, to_project_dir)
+        if not os.path.isdir(from_proto_dir):
+            print(f'\n==> Error - prototype admin project... did you complete setup: https://github.com/valhuber/ApiLogicServer/wiki/Internals')
+            exit(1)
         shutil.copytree(from_proto_dir, to_project_dir)
 
+        os.mkdir(pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "admin"))
+
+        home_js_file = "home.js"  # get the welcome screen
+        if self.mod_gen.nw_db_status in ["nw", "nw+"]:  # explains nw-specific customizations
+            home_js_file = "home_nw.js"
+        home_js = pathlib.Path(self.get_create_from_model_dir()).\
+            joinpath("create_from_model", "templates", home_js_file)
+        to_project_dir = pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "admin")
+        shutil.copyfile(home_js, to_project_dir.joinpath("home.js"))
+
 
 def create(model_creation_services: create_from_model.CreateFromModel):
     """ called by ApiLogicServer CLI -- creates ui/admin application (ui/admin folder, admin.yaml)
