@@ -73,6 +73,11 @@ def fix_generated(code, args):
     if "mysql" in args.url:
         code = code.replace("Numeric", "String")
         code = code.replace(", 'utf8_bin'","")
+    if "mssql" in args.url:
+        bad_import = "from sqlalchemy.dialects.mysql import *"  # prevents safrs bool not iterable
+        line1 = "# coding: utf-8\n"
+        code = code.replace(bad_import,"# " + bad_import)
+        code = code.replace(line1, line1 + bad_import + "\n")
     return code
 
 
@@ -95,7 +100,7 @@ def create_models_from_db(args) -> str:
     generator = CodeGenerator(metadata, args.noindexes, args.noconstraints,
                               args.nojoined, args.noinflect, args.noclasses, args.model_creation_services)
     args.model_creation_services.metadata = generator.metadata
-    generator.render(capture)
+    generator.render(capture)  # generates (preliminary) models in memory
     models_py = capture.getvalue()
     models_py = fix_generated(models_py, args)
     return models_py
