@@ -72,9 +72,9 @@ def python_status():
         show("python --version")
     print("")
     print("Typical commands:")
-    print("  ApiLogicServer create-and-run --project_name=/localhost/docker_project --db_url=")
-    print("  ApiLogicServer run-api        --project_name=/localhost/docker_project")
-    print("  ApiLogicServer run-ui         --project_name=/localhost/docker_project   # login admin, p")
+    print("  ApiLogicServer create-and-run --project_name=/localhost/api_logic_server --db_url=")
+    print("  ApiLogicServer run-api        --project_name=/localhost/api_logic_server")
+    print("  ApiLogicServer run-ui         --project_name=/localhost/api_logic_server   # login admin, p")
     print("  ApiLogicServer sys-info")
     print("  ApiLogicServer version")
     print("")
